package edu.nyu.scps.manhattan;

import android.content.Context;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.graphics.Path;
import android.graphics.PointF;
import android.os.Build;
import android.view.View;


public class ManhattanView extends View {
    private Paint paint = new Paint(Paint.ANTI_ALIAS_FLAG);

    /*
    An array of PointF objects,
    counterclockwise around the shore of Manhattan.
    Longitude west of Greenwich is negative.
    Latitude north of the equator is positive.
    */

    private static PointF[] point = {
            new PointF(-73.971548f,	40.72921f),		//East River at East 17th Street
            new PointF(-73.974595f,	40.735519f),	//24
            new PointF(-73.971806f,	40.742998f),	//34
            new PointF(-73.96215f,	40.754767f),	//53
            new PointF(-73.954296f,	40.762146f),	//65
            new PointF(-73.946185f,	40.771474f),	//81
            new PointF(-73.942022f,	40.776154f),	//89
            new PointF(-73.942022f,	40.776154f),	//96
            new PointF(-73.93816f,	40.787008f),	//103
            new PointF(-73.929534f,	40.795326f),	//118
            new PointF(-73.929062f,	40.800946f),	//125
            new PointF(-73.934212f,	40.808775f),	//Harlem River at 132nd Street
            new PointF(-73.933868f,	40.817772f),	//143
            new PointF(-73.935113f,	40.83547f),		//163
            new PointF(-73.922195f,	40.855857f),	//Dyckman Street
            new PointF(-73.91078f,	40.869878f),	//218
            new PointF(-73.911767f,	40.873416f),	//Broadway Bridge
            new PointF(-73.922968f,	40.877018f),	//Henry Hudson Parkway Bridge
            new PointF(-73.926916f,	40.877082f),	//Hudson River
            new PointF(-73.933096f,	40.867379f),	//Riverside Drive
            new PointF(-73.943224f,	40.852417f),	//Hudson River at West 181st Street
            new PointF(-73.946786f,	40.850339f),	//George Washington Bridge
            new PointF(-73.946786f,	40.850339f),	//168
            new PointF(-73.95052f,	40.834626f),	//155
            new PointF(-73.955026f,	40.827417f),	//144 sewage treatment plant
            new PointF(-73.956399f,	40.828034f),	//144
            new PointF(-73.959446f,	40.82365f),		//137
            new PointF(-73.957601f,	40.822676f),	//137
            new PointF(-73.994765f,	40.771669f),	//57
            new PointF(-73.995152f,	40.769524f),	//54
            new PointF(-73.999872f,	40.763316f),	//44
            new PointF(-74.001718f,	40.762276f),	//42
            new PointF(-74.007726f,	40.754052f),	//29
            new PointF(-74.009442f,	40.749825f),	//23
            new PointF(-74.00794f,	40.748362f),	//21
            new PointF(-74.009228f,	40.740754f),	//Meatpacking District
            new PointF(-74.010344f,	40.739258f),	//Gansevoort Street
            new PointF(-74.011545f,	40.726218f),	//Holland Tunnel
            new PointF(-74.013176f,	40.718315f),	//Battery Park City
            new PointF(-74.016609f,	40.718737f),	//Battery Park City
            new PointF(-74.019227f,	40.706539f),	//South Cove
            new PointF(-74.014893f,	40.70078f),		//Battery Park
            new PointF(-74.009314f,	40.701919f),	//Heliport
            new PointF(-73.997984f,	40.708523f),	//north of Brooklyn Bridge
            new PointF(-73.977985f,	40.710475f),	//Corlears Hook Park
            new PointF(-73.976011f,	40.712752f),	//Grand Street
            new PointF(-73.972964f,	40.720819f)		//East 6th Street
    };

    public ManhattanView(Context context) {
        super(context);
        paint.setColor(Color.RED);
        paint.setStyle(Paint.Style.FILL);

        //Turn off hardware acceleration
        //to prevent Canvas.scale from blurring or diluting the picture.
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.HONEYCOMB) {
            setLayerType(LAYER_TYPE_SOFTWARE, paint);
        }
    }

    @Override
    protected void onDraw(Canvas canvas) {
        super.onDraw(canvas);

        canvas.drawColor(Color.WHITE);
        int width = getWidth();
        int height = getHeight();

        //Three transformations:
        //Transformation 1: move the origin (0, 0) to the center of the View.
        canvas.translate(width / 2, height / 2);

        //Center of Manhattan, near the Angel of the Waters in Central Park.
        PointF center = new PointF(-73.965f, 40.79f);
        double latitude = Math.toRadians(center.y); //latitude in radians

        //The screen will cover 1/4 of a degree of latitude vertically,
        //approx 15 miles.  New York, New York, its a helluva town!

        //pixels per degree of latitude (approx 60 miles)
        float verticalScale = 4 * height;
        //pixels per degree of longitude (approx 45 miles in New York)
        float horizontalScale = verticalScale * (float)Math.cos(latitude);

        //Transformation 2: magnify the map, and make the Y axis point up.
        canvas.scale(horizontalScale, -verticalScale);

        //Transformation 3: move the camera from Latitude 0, Longitude 0
        //(in the South Atlantic) to the center of Manhattan.
        canvas.translate(-center.x, -center.y);

        Path path = new Path();
        path.moveTo(point[0].x, point[0].y);

        for (int i = 1; i < point.length; ++i) {
            path.lineTo(point[i].x, point[i].y);
        }

        path.close();
        canvas.drawPath(path, paint);
    }

}
